  
#ifndef _deconvolver_cmpt_h_
#define _deconvolver_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/synthesis/deconvolver_forward.h>

using namespace std;

namespace casac {
           


   class  deconvolver  {
       public :


         ~deconvolver();
         deconvolver();

         bool open(const  string& dirty=string(""), const  string& psf=string(""),  bool  warn=true);

         bool reopen();

         bool close();

         bool done();

         bool summary();

         bool boxmask(const  string& mask=string(""),  const std::vector<int>& blc=initialize_vector(1, (int)-1),  const std::vector<int>& trc=initialize_vector(1, (int)-1),  const variant & fillvalue= variant(),  const variant & outsidevalue= variant());

         bool regionmask(const  string& mask=string(""),  const record& region=initialize_record("") ,  const variant & boxes= variant(), 
			  double value=1.0);

         bool clipimage(const  string& clippedimage=string(""), const  string& inputimage=string(""),  const variant & threshold= variant());

         bool clarkclean( int  niter=1000, 
			  double gain=0.1,  const variant & threshold= variant(),  bool  displayprogress=false, const  string& model=string(""), const  string& mask=string(""),  int  histbins=500,  const std::vector<int>& psfpatchsize=initialize_vector(2,(int)51, (int)51), 
			  double maxextpsf=0.2, 
			  double speedup=0.0,  int  maxnumpix=10000,  int  maxnummajcycles=-1,  int  maxnumminoriter=-1);

         record* fullclarkclean( int  niter=1000, 
			  double gain=0.1,  const variant & threshold= variant(), const  string& model=string(""), const  string& mask=string(""), 
			  double cyclefactor=1.5);

         string dirtyname();

         string psfname();

         bool make(const  string& image=string(""),  bool  async=false);

         bool convolve(const  string& convolvemodel=string(""), const  string& model=string(""));

         bool makegaussian(const  string& gaussianimage=string(""),  const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  normalize=true,  bool  async=false);

         bool state();

         bool updatestate(const  string& f=string(""), const  string& method=string("INIT"));

         record* clean(const  string& algorithm=string("hogbom"),  int  niter=1000, 
			  double gain=0.1,  const variant & threshold= variant(),  bool  displayprogress=false, const  string& model=string(""), const  string& mask=string(""),  bool  async=false);

         record* naclean( int  niter=1000, 
			  double gain=0.1,  const variant & threshold= variant(), const  string& model=string(""), const  string& mask=string(""),  int  masksupp=3, const  string& memory=string("medium"), 
			  double numsigma=5.0);

         bool setscales(const  string& scalemethod=string("nscales"),  int  nscales=5,  const std::vector<double>& uservector=initialize_vector(3,(double)0.0, (double)3.0, (double)10.0));

         bool ft(const  string& model=string(""), const  string& transform=string(""),  bool  async=false);

         bool restore(const  string& model=string(""), const  string& image=string(""),  const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  async=false);

         bool residual(const  string& model=string(""), const  string& image=string(""),  bool  async=false);

         bool smooth(const  string& model=string(""), const  string& image=string(""),  const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  normalize=true,  bool  async=false);

         bool mem(const  string& entropy=string("entropy"),  int  niter=20,  const variant & sigma= variant(),  const variant & targetflux= variant(),  bool  constrainflux=false,  bool  displayprogress=false, const  string& model=string(""), const  string& prior=string(""), const  string& mask=string(""),  bool  imageplane=false,  bool  async=false);

         bool makeprior(const  string& prior=string(""), const  string& templateimage=string(""),  const variant & lowclipfrom= variant(),  const variant & lowclipto= variant(),  const variant & highclipfrom= variant(),  const variant & highclipto= variant(),  const std::vector<int>& blc=initialize_vector(1, (int)-1),  const std::vector<int>& trc=initialize_vector(1, (int)-1),  bool  async=false);

         bool mtopen( int  ntaylor=2,  const std::vector<double>& scalevector=initialize_vector(3,(double)0.0, (double)3.0, (double)10.0),  const std::vector<std::string>& psfs=std::vector<string>(1, ""),  bool  async=false);

         record* mtclean( const std::vector<std::string>& residuals=std::vector<string>(1, ""),  const std::vector<std::string>& models=std::vector<string>(1, ""),  int  niter=1000, 
			  double gain=0.1,  const variant & threshold= variant(),  bool  displayprogress=false, const  string& mask=string(""),  bool  async=false);

         bool mtrestore( const std::vector<std::string>& models=std::vector<string>(1, ""),  const std::vector<std::string>& residuals=std::vector<string>(1, ""),  const std::vector<std::string>& images=std::vector<string>(1, ""),  const variant & bmaj= variant(),  const variant & bmin= variant(),  const variant & bpa= variant(),  bool  async=false);

         bool mtcalcpowerlaw( const std::vector<std::string>& images=std::vector<string>(1, ""),  const std::vector<std::string>& residuals=std::vector<string>(1, ""), const  string& alphaname=string(""), const  string& betaname=string(""),  const variant & threshold= variant(),  bool  calcerror=false,  bool  async=false);


      private :
#include <tools/synthesis/deconvolver_private.h>
   }; 

 
}
#endif
