  
#ifndef _vpmanager_cmpt_h_
#define _vpmanager_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/synthesis/vpmanager_forward.h>

using namespace std;

namespace casac {
           


   class  vpmanager  {
       public :


         ~vpmanager();
         vpmanager();

         bool saveastable(const  string& tablename=string(""));

         bool loadfromtable(const  string& tablename=string(""));

         bool summarizevps( bool  verbose=false);

         record* setcannedpb(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true, const  string& commonpb=string("DEFAULT"),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         record* setpbairy(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const variant & dishdiam= variant(),  const variant & blockagediam= variant(),  const variant & maxrad= variant(),  const variant & reffreq= variant(),  const variant & squintdir= variant(),  const variant & squintreffreq= variant(),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         record* setpbcospoly(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& coeff=initialize_vector(1, (double)-1),  const std::vector<double>& scale=initialize_vector(1, (double)-1),  const variant & maxrad= variant(),  const variant & reffreq= variant(), const  string& isthispb=string("PB"),  const variant & squintdir= variant(),  const variant & squintreffreq= variant(),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         record* setpbgauss(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const variant & halfwidth= variant(),  const variant & maxrad= variant(),  const variant & reffreq= variant(), const  string& isthispb=string("PB"),  const variant & squintdir= variant(),  const variant & squintreffreq= variant(),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         record* setpbinvpoly(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& coeff=initialize_vector(1, (double)-1),  const variant & maxrad= variant(),  const variant & reffreq= variant(), const  string& isthispb=string("PB"),  const variant & squintdir= variant(),  const variant & squintreffreq= variant(),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         record* setpbnumeric(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& vect=initialize_vector(1, (double)-1),  const variant & maxrad= variant(),  const variant & reffreq= variant(), const  string& isthispb=string("PB"),  const variant & squintdir= variant(),  const variant & squintreffreq= variant(),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         record* setpbimage(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true, const  string& realimage=string(""), const  string& imagimage=string(""), const  string& compleximage=string(""),  const std::vector<std::string>& antnames=std::vector<string>(1, ""));

         record* setpbpoly(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& coeff=initialize_vector(1, (double)-1),  const variant & maxrad= variant(),  const variant & reffreq= variant(), const  string& isthispb=string("PB"),  const variant & squintdir= variant(),  const variant & squintreffreq= variant(),  bool  dosquint=false,  const variant & paincrement= variant(),  bool  usesymmetricbeam=false);

         bool setpbantresptable(const  string& telescope=string(""), const  string& othertelescope=string(""),  bool  dopb=true, const  string& antresppath=string(""));

         bool reset();

         bool setuserdefault( int  vplistnum=-1, const  string& telescope=string(""), const  string& anttype=string(""));

         int getuserdefault(const  string& telescope=string(""), const  string& anttype=string(""));

         std::vector<std::string> getanttypes(const  string& telescope=string(""),  const variant & obstime = variant(),  const variant & freq = variant(),  const variant & obsdirection = variant());

         int numvps(const  string& telescope=string(""),  const variant & obstime = variant(),  const variant & freq = variant(),  const variant & obsdirection = variant());

         record* getvp(const  string& telescope=string(""), const  string& antennatype=string(""),  const variant & obstime = variant(),  const variant & freq = variant(),  const variant & obsdirection = variant());

         record* getvps(const  string& telescope=string(""),  const std::vector<std::string>& antennas=std::vector<string>(1, ""),  const variant & obstimestart = variant(),  const variant & obstimeend = variant(),  const variant & minfreq = variant(),  const variant & maxfreq = variant(),  const variant & obsdirection = variant());

         bool createantresp( const string& imdir=string(""),  const string& starttime=string(""),  const std::vector<std::string>& bandnames=std::vector<string>(1, ""),  const std::vector<std::string>& bandminfreq=std::vector<string>(1, ""),  const std::vector<std::string>& bandmaxfreq=std::vector<string>(1, ""));

         string getrespimagename( const string& telescope=string(""),  const string& starttime=string(""), const  string& frequency=string(""), const  string& functype=string("ANY"), const  string& anttype=string(""), const  string& azimuth=string("0deg"), const  string& elevation=string("45deg"), const  string& rectype=string(""),  int  beamnumber=0);


      private :
#include <tools/synthesis/vpmanager_private.h>
   }; 

 
}
#endif
